//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "ui_mainwindow.h"
#include <QMainWindow>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QtXml/QDomDocument>
#include <QLineEdit>
#include <QPlainTextEdit>
#include <QCheckBox>
#include <QLabel>
#include <QFileDialog>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QtDebug>
#include <QProgressDialog>
#include "models/personalinfo.h"
#include "models/subjects.h"
#include "models/skills.h"
#include "models/workexperiences.h"
#include "models/educationdegrees.h"
#include "models/certificates.h"
#include "models/publications.h"
#include "models/languages.h"
#include "models/templatetranslates.h"
#include "models/technicalsynonyms.h"
#include "models/coverletter.h"
#include "models/jobapplications.h"
#include "models/resumedocument.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    enum JobApplicationState {
        Unknown,
        OnGoing,
        PendingMeCritical,
        PendingThemCritical,
        Abandoned
    };

    explicit MainWindow(QWidget *parent = 0,QString filePath=QString());
    ~MainWindow();

private:
    Ui::MainWindow *ui;
    QString filePath;
    bool modified;
    QGraphicsScene *gsProfilePhoto;
    bool profilePhotoFit;
    QPixmap pmProfilePhoto;
    QDomDocument domDocument;
    QFileDialog fileDialog;
    PersonalInfo personalInfo;
    Subjects subjects;
    Skills skills;
    WorkExperiences workExperiences;
    EducationDegrees educationDegrees;
    Certificates certificates;
    Publications publications;
    Languages languages;
    TemplateTranslates templateTranslates;
    TechnicalSynonyms technicalSynonyms;
    CoverLetter coverLetter;
    JobApplications jobApplications;
    QStringList RequiredSkills;
    int DefaultFontSize;
    void open(QString filePath);
    void setCheckBoxes(bool checked);
public slots:
    void onModified();
private slots:
    void on_pbZoom_clicked();
    void on_pbRemoveProfilePhoto_clicked();
    void on_pbAddProfilePhoto_clicked();
    bool on_actionNew_triggered();
    void showEvent(QShowEvent *event);
    void closeEvent(QCloseEvent *event);
    void on_actionOpen_triggered();
    bool on_actionSave_triggered();
    bool on_actionSave_as_triggered();
    void on_actionExit_triggered();
    void on_actionAbout_triggered();
    void on_action_Export_as_word_2007_docx_triggered();
    void on_actionE_xport_Coverl_Letter_triggered();
    void on_actionResume_Analyser_triggered();
    void on_actionAnalyse_triggered();
    void on_pbAnalyserSaveItems_clicked();
    void on_actionSelect_a_ll_triggered();
    void on_actionSelect_none_triggered();
    void on_actionHelp_triggered();
    void on_actionAbout_Qt_triggered();
    void on_actionSet_export_font_size_triggered();
    void on_action_Header_triggered();
    void on_action_Subjects_triggered();
    void on_action_Skills_triggered();
    void on_action_Work_Experiences_triggered();
    void on_actionEducation_Degrees_triggered();
    void on_actionCertificates_triggered();
    void on_action_Publications_triggered();
    void on_action_Languages_triggered();
    void on_action_Edit_Required_Skills_triggered();
    void on_action_Un_Fold_all_triggered();
    void on_rbJobApplicationsAll_clicked();
    void on_rbJobApplicationsOnGoing_clicked();
    void on_rbJobApplicationsCritical_clicked();
    void on_actionArchive_Abandoned_Job_Applications_triggered();
    void on_action_Donate_triggered();
    void on_pbJumpToTop_clicked();
};

#endif // MAINWINDOW_H
