//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "templator.h"
#include <QFile>
#include <QTextStream>

Templator::Templator()
{
}

QString Templator::readTemplate(QString templateName, QList<Translate*> templatetranslates)
{
    QFile f(":/templates/resources/templates/"+templateName+".xml");
    f.open(QFile::ReadOnly | QFile::Text);
    QTextStream ts(&f);
    //Here we want to translate our template into user's language
    //For this we will replace occurances of user defined phrases with user defined translations
    //But we don't want to touch text between brackets as these are our variables
    //So we split our template by closed brackets so each part only contains 1 open bracket
    QStringList parts=ts.readAll().split("]");
    QStringList translated_parts;
    for (int i=0; i<parts.count(); i++) {
        //Then for each part we split again this time based on open bracket
        QStringList parts_parts=parts.at(i).split("[");
        for (int j=0; j<templatetranslates.count(); j++)
            //Thenjust take left side of that open bracket and translate it
            parts_parts.first().replace(templatetranslates.at(j)->from().trimmed(),templatetranslates.at(j)->to().trimmed().toHtmlEscaped(),Qt::CaseInsensitive);
        //At the end we glue translated and non-translated parts back together with an open bracket
        //And add them to a new list of parts
        translated_parts.append(parts_parts.join("["));
    }
    //Now we want to join all our translated parts back together with closed brackets
    //But if last parts contains an open bracket it is because the last closed bracket was at the end of
    //Our template and if we just joins them, the last open bracket will not get closed
    if (translated_parts.last().contains("["))
        //So in this case we add a closed bracket at the end
        translated_parts.last().append("]");
    //Finally we glue them and return the result
    return translated_parts.join("]");
}

