//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#ifndef FIELD_H
#define FIELD_H

#include <QList>
#include <QString>
#include <QCheckBox>
#include <QToolButton>
#include <QDateEdit>

class SubgroupInput;
class FieldInput;

class Field
{
public:
    enum FieldType {
        Line,
        Paragraph,
        MultiLine,
        MultiParagraph,
        Date,
        Combo,
        MultiDatedCombo
    };
private:
    QString m_name;
    FieldType m_type;
    QString m_title;
    QString m_defaultText;
    bool m_updatesGroupTitle;
    bool m_selectable;
    bool m_itemsSelectable;
public:
    Field(QString name, FieldType type, QString title=NULL, QString defaultText=NULL, bool updatesGroupTitle=false, bool selectable=false, bool itemsSelectable=true);
    QString name() {return this->m_name;}
    FieldType type() {return this->m_type;}
    QString title() {return this->m_title;}
    QString defaultText() {return this->m_defaultText;}
    bool updatesGroupTitle() {return this->m_updatesGroupTitle;}
    bool selectable() {return this->m_selectable;}
    bool itemsSelectable() {return this->m_itemsSelectable;}
};

class FieldInputItem
{
public:
    FieldInput *fieldInput;
    QWidget *input;
    QCheckBox* include;
    QWidget *containerWidget;
    QDateEdit *dateEdit;
    FieldInputItem(FieldInput *fieldInput, QWidget *input, QCheckBox* include, QWidget *containerWidget, QDateEdit *dateEdit=NULL) : fieldInput(fieldInput), input(input), include(include), containerWidget(containerWidget), dateEdit(dateEdit) {}
};

class FieldInput
{
public:
    SubgroupInput *subgroupInput;
    QList<FieldInputItem*> items;
    QWidget *input;
    QCheckBox *include;
    Field *field;
    FieldInput(SubgroupInput *subgroupInput, Field *field) : subgroupInput(subgroupInput), field(field) {}
};

class FieldList: public QList<Field*>
{
public:
    Field *fieldByName(QString name);
};

#endif // FIELD_H
