//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "fieldsgroup.h"
#include "multiitemfield.h"
#include "inputfield.h"
#include <QApplication>

#define MAX_FOLDED_TITLE_SIZE   100

FieldsGroup::FieldsGroup(QWidget *parent, QString name, QString title, bool selectable, bool movable) :
    QGroupBox(parent),
    m_title(title),
    m_selectable(selectable),
    m_movable(movable)
{
    setObjectName(name);
    checkBox.setChecked(true);
    checkBox.setText("Include this part");
    connect(&checkBox,SIGNAL(clicked()),this,SIGNAL(modified()));
    if (m_selectable)
        hlayout.addWidget(&checkBox);
    else {
        hlayout.addWidget(&foldLabel);
        foldLabel.setVisible(false);//When group folded this will become visible
    }
    setTitle(title);
    hlayout.addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    insertButton.setToolTip("Insert");
    insertButton.setStatusTip("Insert another part before this");
    insertButton.setIcon(QIcon(":/resources/icons/resources/icons/window-new.png"));
    insertButton.setIconSize(QSize(16,16));
    connect(&insertButton,SIGNAL(clicked()),this,SLOT(onInsertButtonClicked()));
    if (movable)
        hlayout.addWidget(&this->insertButton);
    downButton.setToolTip("Move down");
    downButton.setStatusTip("Move one item down");
    downButton.setIcon(QIcon(":/resources/icons/resources/icons/go-down.png"));
    downButton.setIconSize(QSize(16,16));
    connect(&downButton,SIGNAL(clicked()),this,SLOT(onDownButtonClicked()));
    if (movable)
        hlayout.addWidget(&downButton);
    deleteButton.setToolTip("Delete item");
    deleteButton.setStatusTip("Delete item");
    deleteButton.setIcon(QIcon(":/resources/icons/resources/icons/list-remove.png"));
    deleteButton.setIconSize(QSize(16,16));
    connect(&deleteButton,SIGNAL(clicked()),this,SLOT(onDeleteButtonClicked()));
    if (movable)
        hlayout.addWidget(&deleteButton);
    upButton.setToolTip("Move up");
    upButton.setStatusTip("Move one item up");
    upButton.setIcon(QIcon(":/resources/icons/resources/icons/go-up.png"));
    upButton.setIconSize(QSize(16,16));
    connect(&upButton,SIGNAL(clicked()),this,SLOT(onUpButtonClicked()));
    if (movable)
        hlayout.addWidget(&upButton);
    foldButton.setText("^");
    foldButton.setToolTip("Fold this part");
    foldButton.setStatusTip("Fold this part");
    foldButton.setIcon(QIcon(":/resources/icons/resources/icons/extract-archive.png"));
    foldButton.setIconSize(QSize(16,16));
    connect(&foldButton,SIGNAL(clicked()),this,SLOT(onFoldButtonClicked()));
    hlayout.addWidget(&foldButton);
    QWidget *hwidget=new QWidget();
    hwidget->setLayout(&hlayout);
    vlayout.addWidget(hwidget);
    QWidget *fieldsWidget=new QWidget(this);
    fieldsWidget->setLayout(&fieldsLayout);
    vlayout.addWidget(fieldsWidget);
    setLayout(&vlayout);
}

QDomElement FieldsGroup::toDomElement(QDomDocument *document)
{
    QDomElement deFieldGroup=document->createElement(m_movable?"group":objectName());//If it's not movable then it's not a subgroup
    if (m_selectable) {
        deFieldGroup.setAttribute("include",checkBox.isChecked()?"1":"0");
    }

    for (int i=0; i<fieldsLayout.count(); i++) {
        InputField *field=static_cast<InputField*>(fieldsLayout.itemAt(i)->widget());
        deFieldGroup.appendChild(field->toDomElement(document));
    }
    return deFieldGroup;
}

void FieldsGroup::fromDomElement(QProgressDialog *pd, QDomElement element)
{
    checkBox.setChecked(element.attribute("include","1")=="1");
    for (int i=0; i<fieldsLayout.count(); i++) {
        InputField *field=static_cast<InputField*>(fieldsLayout.itemAt(i)->widget());
        QDomNodeList nodes=element.elementsByTagName(field->objectName());
        if (nodes.count()) {
            field->fromDomElement(nodes.at(0).toElement());
        }
        else
            field->clear();
    }
    pd->setValue(pd->value()+1);
    qApp->processEvents();
}

void FieldsGroup::clear()
{
    for (int i=0; i<fieldsLayout.count(); i++) {
        InputField *field=static_cast<InputField*>(fieldsLayout.itemAt(i)->widget());
        field->clear();
    }
}

void FieldsGroup::fold(bool fold)
{
    for (int i=1; i<vlayout.count(); i++)
        vlayout.itemAt(i)->widget()->setVisible(!fold);
    if (fold) {
        QString foldedTitle;
        for (int i=0; i<fieldsLayout.count(); i++) {
            InputField *inputField=static_cast<InputField*>(fieldsLayout.itemAt(i)->widget());
            if (!inputField->updatesGroupTitle())
                continue;
            QString text=inputField->text().trimmed();

            if ((foldedTitle.count()) && (text.count()))
                foldedTitle+="; ";
            foldedTitle+=text;
            if (foldedTitle.count()>MAX_FOLDED_TITLE_SIZE) {
                foldedTitle=foldedTitle.left(MAX_FOLDED_TITLE_SIZE)+"...";
                break;
            }
        }
        if (foldedTitle.trimmed().isEmpty())
            foldedTitle="...";
        if (m_selectable)
            checkBox.setText(foldedTitle);
        else {
            foldLabel.setText(foldedTitle);
            foldLabel.setVisible(true);
        }
    }
    else {//Unfold
        if (m_selectable)
            checkBox.setText("Include this part");
        else
            foldLabel.setVisible(false);
    }
    foldButton.setText(fold?"...":"^");
    foldButton.setToolTip(fold ? "Collapse this part" : "Fold this part");
    foldButton.setStatusTip(foldButton.toolTip());
    foldButton.setIcon(
                QIcon(
                    fold ?
                        ":/resources/icons/resources/icons/expand-archive.png"
                      : ":/resources/icons/resources/icons/extract-archive.png"
                        )
                );
}

QString FieldsGroup::exportDocX(QList<Translate *> templatetranslates)
{
    QString result=Templator::readTemplate(objectName(), templatetranslates);
    for (int i=0; i<fieldsLayout.count(); i++) {
        InputField *field=static_cast<InputField*>(fieldsLayout.itemAt(i)->widget());
        if (!field->isChecked())
            continue;
        MultiItemField *multiField=qobject_cast<MultiItemField*>(field);
        QString t;
        if (multiField) {
            QString name=objectName()+"_"+field->objectName()+"_item";
            for (int i=0; i<multiField->itemsLayout.count(); i++) {
                InputItem* item=static_cast<InputItem*>(multiField->itemsLayout.itemAt(i)->widget());
                if (!item->isChecked())
                    continue;
                t+=Templator::readTemplate(name,templatetranslates).replace("["+name+"]",item->text().toHtmlEscaped());
            }
        }
        else
            t=field->text().toHtmlEscaped();
        result.replace("["+objectName()+"_"+field->objectName()+"]",t);
    }
    return result;
}

int FieldsGroup::analyze(QString skill, QList<QStringList> technicalSynonyms, bool autoselect, bool autoreplace, bool ask)
{
    if (m_selectable) {
        if ((!autoselect) && (!checkBox.isChecked()))
            return 0;
    }
    int count=0;
    for (int i=0; i<fieldsLayout.count(); i++) {
        MultiItemField *multiItemField=qobject_cast<MultiItemField*>(fieldsLayout.itemAt(i)->widget());
        if (multiItemField)
            count+=multiItemField->analyze(skill,technicalSynonyms,autoselect,autoreplace,ask);
        else {
            InputField *field=static_cast<InputField*>(fieldsLayout.itemAt(i)->widget());
            count+=field->analyze(skill,technicalSynonyms,autoselect,autoreplace,ask);
        }
    }
    if ((count) && (m_selectable) && (autoselect) && (!checkBox.isChecked()))
        checkBox.setChecked(true);

    return count;
}

void FieldsGroup::addField(InputField* field)
{
    fieldsLayout.addWidget(field);
    connect(field,SIGNAL(modified()),this,SIGNAL(modified()));
}

void FieldsGroup::onUpButtonClicked()
{
    int index=this->parentWidget()->layout()->indexOf(this);
    if (index>0) {
        QVBoxLayout *parentLayout=static_cast<QVBoxLayout*>(this->parentWidget()->layout());
        parentLayout->removeWidget(this);
        parentLayout->insertWidget(index-1,this);
    }
    emit modified();
}

void FieldsGroup::onDownButtonClicked()
{
    int index=this->parentWidget()->layout()->indexOf(this);
    if (index<this->parentWidget()->layout()->count()-1) {
        QVBoxLayout *parentLayout=static_cast<QVBoxLayout*>(this->parentWidget()->layout());
        parentLayout->removeWidget(this);
        parentLayout->insertWidget(index+1,this);
    }
    emit modified();
}

void FieldsGroup::onFoldButtonClicked()
{
    bool mustFold=foldButton.text()=="^";
    fold(mustFold);
}

void FieldsGroup::onInsertButtonClicked()
{
    emit this->insertClicked(parentWidget()->layout()->indexOf(this));
    emit modified();
}

void FieldsGroup::onDeleteButtonClicked()
{
    this->deleteButton.setVisible(false);
    this->setVisible(false);
    this->parentWidget()->layout()->removeWidget(this);
    emit modified();
    qApp->processEvents();
    this->deleteLater();
}
