//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "fieldsgroupslayout.h"
#include <QApplication>
#include <QDebug>

FieldsGroupsLayout::FieldsGroupsLayout(QWidget *parent, QString description, bool movable, bool itemSelectable) :
    QVBoxLayout(parent),
    m_movable(movable),
    m_itemSelectable(itemSelectable)
{
  if (m_movable) {
      checkBox.setChecked(true);
      connect(&checkBox,SIGNAL(clicked()),this,SIGNAL(modified()));
      hlayout.addWidget(&checkBox);
      downButton.setToolTip("Move down");
      downButton.setStatusTip("Move one item down");
      downButton.setIcon(QIcon(":/resources/icons/resources/icons/go-down.png"));
      downButton.setIconSize(QSize(24, 24));
      connect(&downButton,SIGNAL(clicked()),this,SIGNAL(downButtonClicked()));
      hlayout.addWidget(&downButton);
      upButton.setToolTip("Move up");
      upButton.setStatusTip("Move one item up");
      upButton.setIcon(QIcon(":/resources/icons/resources/icons/go-up.png"));
      upButton.setIconSize(QSize(24, 24));
      connect(&upButton,SIGNAL(clicked()),this,SIGNAL(upButtonClicked()));
      hlayout.addWidget(&upButton);
      foldButton.setText("^");
      foldButton.setToolTip("Fold this section");
      foldButton.setStatusTip("Fold this section");
      foldButton.setIcon(QIcon(":/resources/icons/resources/icons/extract-archive.png"));
      foldButton.setIconSize(QSize(24, 24));
      connect(&foldButton,SIGNAL(clicked()),this,SLOT(onFoldButtonClicked()));
      hlayout.addWidget(&foldButton);
      QWidget *hwidget=new QWidget();
      hwidget->setLayout(&hlayout);
      addWidget(hwidget);
  }
  if (description != NULL)
      addWidget(new QLabel(description,parentWidget()));
  QWidget *fieldgroupsWidget=new QWidget(parentWidget());
  fieldgroupsWidget->setLayout(&subgroupsLayout);
  addWidget(fieldgroupsWidget);
  QWidget *buttonWidget=new QWidget();
  QHBoxLayout *buttonLayout=new QHBoxLayout(buttonWidget);
  buttonLayout->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
  addButton.setToolTip("Add new");
  addButton.setStatusTip("Add new part to this section");
  addButton.setIcon(QIcon(":/resources/icons/resources/icons/window-new.png"));
  addButton.setIconSize(QSize(24, 24));
  buttonLayout->addWidget(&addButton);
  connect(&addButton,SIGNAL(clicked()),this,SIGNAL(addButtonClicked()));
  buttonLayout->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
  buttonWidget->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::MinimumExpanding);
  addWidget(buttonWidget);
}

QDomElement FieldsGroupsLayout::toDomElement(QDomDocument *document)
{
    QDomElement deGroup=document->createElement(parentWidget()->objectName());
    if (m_movable)
        deGroup.setAttribute("include",checkBox.isChecked()?"1":"0");
    for (int i=0; i<subgroupsLayout.count(); i++) {
        FieldsGroup *item=qobject_cast<FieldsGroup*>(subgroupsLayout.itemAt(i)->widget());
        deGroup.appendChild(item->toDomElement(document));
    }
    return deGroup;
}

void FieldsGroupsLayout::fromDomElement(QProgressDialog *pd, QDomElement element)
{
    checkBox.setChecked(element.attribute("include","1")=="1");
    for (int i=0; i<subgroupsLayout.count(); i++)
        subgroupsLayout.itemAt(i)->widget()->deleteLater();
    qApp->processEvents();
    QDomNodeList itemNodes=element.childNodes();
    for (int i=0; i<itemNodes.count(); i++) {
        emit addButtonClicked();
        qApp->processEvents();
        FieldsGroup *item=qobject_cast<FieldsGroup*>(subgroupsLayout.itemAt(subgroupsLayout.count()-1)->widget());
        item->fromDomElement(pd,itemNodes.at(i).toElement());
    }
}

QString FieldsGroupsLayout::exportDocX(QList<Translate *> templatetranslates)
{
    if ((m_movable) && (!checkBox.isChecked()))
        return QString();
    QString result=Templator::readTemplate(parentWidget()->objectName(), templatetranslates);
    for (int i=0; i<subgroupsLayout.count(); i++) {
        FieldsGroup *fieldsGroup=qobject_cast<FieldsGroup*>(subgroupsLayout.itemAt(i)->widget());
        if (!fieldsGroup->checkBox.isChecked())
            continue;
        result+=fieldsGroup->exportDocX(templatetranslates);
    }
    return result;
}

void FieldsGroupsLayout::clear()
{
    while (subgroupsLayout.count())
        delete subgroupsLayout.itemAt(0)->widget();
    qApp->processEvents();
}

void FieldsGroupsLayout::fold(bool fold)
{
    if (!m_movable)
        return;
    for (int i=1; i<layout()->count(); i++)
        layout()->itemAt(i)->widget()->setVisible(!fold);
    foldButton.setText(fold?"...":"^");
    foldButton.setToolTip(fold ? "Collapse this part" : "Fold this part");
    foldButton.setStatusTip(foldButton.toolTip());
    foldButton.setIcon(
                QIcon(
                    fold ?
                        ":/resources/icons/resources/icons/expand-archive.png"
                      : ":/resources/icons/resources/icons/extract-archive.png"
                        )
                );
}

int FieldsGroupsLayout::analyze(QString skill, QList<QStringList> technicalSynonyms, bool autoselect, bool autoreplace, bool ask)
{
    if ((!autoselect) && (!checkBox.isChecked()))
        return 0;
    int count=0;

    for (int i=0; i<subgroupsLayout.count(); i++) {
        FieldsGroup *fieldsGroup=static_cast<FieldsGroup*>(subgroupsLayout.itemAt(i)->widget());
        count+=fieldsGroup->analyze(skill,technicalSynonyms,autoselect,autoreplace,ask);
    }

    if ((count) && (autoselect) && (!checkBox.isChecked()))
        checkBox.setChecked(true);

    return count;
}

void FieldsGroupsLayout::onFoldButtonClicked()
{
    bool mustFold=foldButton.text()=="^";
    fold(mustFold);
}
