//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "inputfield.h"
#include <QLabel>
#include <QDebug>

InputField::InputField(QWidget *parent, QString name, QString title, QString defaultText, bool updatesGroupTitle, bool selectable) :
    AnalyzableWidget(parent,selectable),
    m_title(title),
    m_defaultText(defaultText),
    m_updatesGroupTitle(updatesGroupTitle)
{
    setObjectName(name);
    if (this->selectable())
        hlayout.addWidget(&checkBox);
    if (m_title!=NULL)
        hlayout.addWidget(new QLabel(m_title+":"));
    setLayout(&hlayout);
}

void InputField::clear()
{
    qWarning() << "clear() is not implemented for class: " << metaObject()->className();
}

QDomElement InputField::toDomElement(QDomDocument *document)
{
    QDomElement element=mainElement(document);
    if (selectable()) {
        element.appendChild(checkedElement(document));
    }
    element.appendChild(entryElement(document));
    return element;
}

QDomElement InputField::mainElement(QDomDocument *document)
{
    return document->createElement(objectName());
}

QDomElement InputField::checkedElement(QDomDocument *document)
{
    QDomElement deChecked=document->createElement("checked");
    deChecked.appendChild(document->createTextNode(checkBox.isChecked()?"1":"0"));
    return deChecked;
}

QDomElement InputField::entryElement(QDomDocument *document)
{
    QDomElement deEntry=document->createElement("entry");
    deEntry.appendChild(document->createTextNode(text()));
    return deEntry;
}

void InputField::fromDomElement(QDomElement element)
{
    QDomNodeList checkeds=element.elementsByTagName("checked");
    if (checkeds.count())
        checkBox.setChecked(checkeds.at(0).toElement().text()=="1");
    else
        checkBox.setChecked(true);
    QDomNodeList entries=element.elementsByTagName("entry");
    if (entries.count())
        processEntryElement(entries.at(0).toElement());
    else
        clear();
}

void InputField::processEntryElement(QDomElement element)
{
    setText(element.text());
}
