//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "inputitem.h"
#include <QVBoxLayout>
#include <QDebug>

InputItem::InputItem(QWidget *parent, QString name, bool selectable) :
    AnalyzableWidget(parent,selectable)
{
    if (this->selectable())
        hlayout.addWidget(&checkBox);
    insertButton.setToolTip("Insert another before this");
    insertButton.setIcon(QIcon(":/resources/icons/resources/icons/list-add.png"));
    insertButton.setIconSize(QSize(16,16));
    connect(&insertButton,SIGNAL(clicked()),this,SLOT(onInsertButtonClicked()));
    hlayout.addWidget(&this->insertButton);
    downButton.setToolTip("Move down");
    downButton.setStatusTip("Move one item down");
    downButton.setIcon(QIcon(":/resources/icons/resources/icons/go-down.png"));
    downButton.setIconSize(QSize(16,16));
    connect(&downButton,SIGNAL(clicked()),this,SLOT(onDownButtonClicked()));
    hlayout.addWidget(&downButton);
    deleteButton.setToolTip("Delete item");
    deleteButton.setStatusTip("Delete item");
    deleteButton.setIcon(QIcon(":/resources/icons/resources/icons/list-remove.png"));
    deleteButton.setIconSize(QSize(16,16));
    connect(&deleteButton,SIGNAL(clicked()),this,SLOT(onDeleteButtonClicked()));
    hlayout.addWidget(&deleteButton);
    upButton.setToolTip("Move up");
    upButton.setStatusTip("Move one item up");
    upButton.setIcon(QIcon(":/resources/icons/resources/icons/go-up.png"));
    upButton.setIconSize(QSize(16,16));
    connect(&upButton,SIGNAL(clicked()),this,SLOT(onUpButtonClicked()));
    hlayout.addWidget(&upButton);
    this->setLayout(&hlayout);
    this->setObjectName(name);
}

void InputItem::clear()
{
    qWarning() << "clear() is not implemented for class: " << metaObject()->className();
}

QDomElement InputItem::toDomElement(QDomDocument *document)
{
    QDomElement element=document->createElement(objectName());
    if (selectable()) {
        QDomElement deChecked=document->createElement("checked");
        deChecked.appendChild(document->createTextNode(checkBox.isChecked()?"1":"0"));
        element.appendChild(deChecked);
    }
    QDomElement deValue=document->createElement("value");
    deValue.appendChild(document->createTextNode(text()));
    element.appendChild(deValue);
    return element;
}

void InputItem::fromDomElement(QDomElement element)
{
    QDomNodeList checkeds=element.elementsByTagName("checked");
    if (checkeds.count())
        checkBox.setChecked(checkeds.at(0).toElement().text()=="1");
    else
        checkBox.setChecked(true);
    QDomNodeList values=element.elementsByTagName("value");
    if (values.count())
        setText(values.at(0).toElement().text());
}

void InputItem::onUpButtonClicked()
{
    int index=this->parentWidget()->layout()->indexOf(this);
    if (index>0) {
        QVBoxLayout *parentLayout=qobject_cast<QVBoxLayout*>(this->parentWidget()->layout());
        parentLayout->removeWidget(this);
        parentLayout->insertWidget(index-1,this);
    }
}

void InputItem::onDownButtonClicked()
{
    int index=this->parentWidget()->layout()->indexOf(this);
    if (index<this->parentWidget()->layout()->count()-1) {
        QVBoxLayout *parentLayout=qobject_cast<QVBoxLayout*>(this->parentWidget()->layout());
        parentLayout->removeWidget(this);
        parentLayout->insertWidget(index+1,this);
    }
}

void InputItem::onDeleteButtonClicked()
{
    deleteLater();
}

void InputItem::onInsertButtonClicked()
{
    emit insertClicked(parentWidget()->layout()->indexOf(this));
}
