//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "datedcomboitem.h"

DatedComboItem::DatedComboItem(QWidget *parent, QStringList comboItems, bool selectable) :
    InputItem(parent,"datedcombo", selectable)
{
    edit.addItems(comboItems);
    edit.setEditable(true);
    hlayout.insertWidget(selectable?1:0,&edit);
    connect(&edit,SIGNAL(currentTextChanged(QString)),this,SIGNAL(modified()));
    dateEdit.setToolTip(DATE_FORMAT);
    dateEdit.setDate(QDate::currentDate());
    hlayout.insertWidget(selectable?1:0,&dateEdit);
    connect(&dateEdit,SIGNAL(dateChanged(QDate)),this,SIGNAL(modified()));
}


QString DatedComboItem::text()
{
    return dateEdit.date().toString(DATE_FORMAT)+"\t"+edit.currentText();
}

void DatedComboItem::setText(QString text)
{
    edit.setCurrentIndex(-1);
    for (int i=0; i<edit.count(); i++)
        if (text.indexOf(edit.itemText(i))==0)
            edit.setCurrentIndex(i);
    edit.setCurrentText(text);
}

QDomElement DatedComboItem::toDomElement(QDomDocument *document)
{
    QDomElement element=document->createElement(objectName());
    if (selectable()) {
        QDomElement deChecked=document->createElement("checked");
        deChecked.appendChild(document->createTextNode(checkBox.isChecked()?"1":"0"));
        element.appendChild(deChecked);
    }
    QDomElement deValue=document->createElement("value");
    deValue.appendChild(document->createTextNode(edit.currentText()));
    element.appendChild(deValue);
    QDomElement deDate=document->createElement("date");
    deDate.appendChild(document->createTextNode(dateEdit.date().toString(DATE_FORMAT)));
    element.appendChild(deDate);
    return element;
}

void DatedComboItem::fromDomElement(QDomElement element)
{
    InputItem::fromDomElement(element);
    QDomNodeList deDates=element.elementsByTagName("date");
    if (deDates.count())
        dateEdit.setDate(QDate::fromString(deDates.at(0).toElement().text(),DATE_FORMAT));
}
