//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "multiitemfield.h"
#include <QApplication>
#include <QDebug>

MultiItemField::MultiItemField(QWidget *parent,
                               QString name,
                               QString title,
                               QString defaultText,
                               bool updatesGroupTitle,
                               bool selectable,
                               bool itemSelectable):
    InputField(parent, name, title, defaultText, updatesGroupTitle, selectable),
    m_itemSelectable(itemSelectable)
{
    QWidget *itemsWidget=new QWidget(&itemsAndButtonWidget);
    itemsWidget->setLayout(&itemsLayout);
    QVBoxLayout *itemsAndButtonLayout= new QVBoxLayout(&itemsAndButtonWidget);
    itemsAndButtonLayout->addWidget(itemsWidget);
    itemsAndButtonWidget.setLayout(itemsAndButtonLayout);
    QWidget *buttonWidget=new QWidget(&itemsAndButtonWidget);
    QHBoxLayout *buttonLayout=new QHBoxLayout(buttonWidget);
    buttonWidget->setLayout(buttonLayout);
    buttonLayout->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    addButton.setToolTip("Add item");
    addButton.setIcon(QIcon(":/resources/icons/resources/icons/list-add.png"));
    addButton.setIconSize(QSize(16,16));
    buttonLayout->addWidget(&addButton);
    connect(&addButton,SIGNAL(clicked()),this,SLOT(on_addButton_clicked()));
    buttonLayout->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    buttonWidget->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::MinimumExpanding);
    itemsAndButtonLayout->addWidget(buttonWidget);
    layout()->addWidget(&itemsAndButtonWidget);
}

QStringList MultiItemField::texts(bool filterByChecked)
{
    QStringList result;
    for (int i=0; i<itemsLayout.count(); i++) {
        InputItem *item=static_cast<InputItem*>(itemsLayout.itemAt(i)->widget());
        if ((filterByChecked) && (!item->checkBox.isChecked()))
            continue;
        result.append(item->text());
    }
    return result;
}

QString MultiItemField::text()
{
    return texts().join(", ");
}

void MultiItemField::setText(QString text)
{
    qWarning() << "setText is not support for MultiItemField" << text;
}

void MultiItemField::clear()
{
    for (int i=0; i<itemsLayout.count(); i++)
        itemsLayout.itemAt(i)->widget()->deleteLater();
}

QDomElement MultiItemField::entryElement(QDomDocument *document)
{
    QDomElement deEntry=document->createElement("entry");
    for (int i=0; i<itemsLayout.count(); i++) {
        InputItem *item=static_cast<InputItem*>(itemsLayout.itemAt(i)->widget());
        deEntry.appendChild(item->toDomElement(document));
    }
    return deEntry;
}

void MultiItemField::processEntryElement(QDomElement element)
{
    clear();
    qApp->processEvents();
    QDomNodeList items=element.childNodes();
    for (int i=0; i<items.count(); i++) {
        InputItem *item=newItem();
        item->fromDomElement(items.at(i).toElement());
        itemsLayout.addWidget(item);
    }
}

InputItem *MultiItemField::makeItem()
{
    qWarning() << "clear() is not implemented for class: " << metaObject()->className();
    return NULL;
}

int MultiItemField::analyze(QString skill, QList<QStringList> technicalSynonyms, bool autoselect, bool autoreplace, bool ask)
{
    if (selectable()) {
        if ((!autoselect) && (!checkBox.isChecked()))
            return 0;
    }
    int count=0;

    for (int i=0; i<itemsLayout.count(); i++) {
        InputItem *item=static_cast<InputItem*>(itemsLayout.itemAt(i)->widget());
        count+=item->analyze(skill,technicalSynonyms,autoselect,autoreplace,ask);
    }

    if ((count) && (selectable()) && (autoselect) && (!checkBox.isChecked()))
        checkBox.setChecked(true);

    return count;
}

InputItem *MultiItemField::newItem()
{
    InputItem *item=makeItem();
    connect(item,SIGNAL(insertClicked(int)),this,SLOT(on_insertButton_clicked(int)));
    connect(item,SIGNAL(modified()),this,SIGNAL(modified()));
    return item;
}

void MultiItemField::on_addButton_clicked()
{
    itemsLayout.addWidget(newItem());
    emit modified();
}

void MultiItemField::on_insertButton_clicked(int index)
{
    itemsLayout.insertWidget(index,newItem());
    emit modified();
}
