//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "analyzablewidget.h"
#include <QDebug>
#include <QMessageBox>

AnalyzableWidget::AnalyzableWidget(QWidget *parent, bool selectable) :
    QWidget(parent),
    m_selectable(selectable)
{
    checkBox.setChecked(true);
    connect(&checkBox,SIGNAL(clicked()),this,SIGNAL(modified()));
}

QString AnalyzableWidget::text()
{
    qWarning() << "text() is not implemented for class: " << metaObject()->className();
    return QString();
}

void AnalyzableWidget::setText(QString text)
{
    qWarning() << "setText() is not implemented for class: " << metaObject()->className() << "\n" << text;
}

int AnalyzableWidget::analyze(QString skill, QList<QStringList> technicalSynonyms, bool autoselect, bool autoreplace, bool ask)
{
    if (m_selectable) {
        if ((!autoselect) && (!checkBox.isChecked()))
            return 0;
    }
    int count=0;
    QRegExp regexp("[^a-zA-Z0-9+]{1}"//Whole words only (At leas one non-alphanumeric character needed
                   +skill
                   .trimmed()
                   .replace("+","[+]{1}")//Plus signs are a pain in the a**! You have to consider them because lots of people have C/C++ in their resume!
                   .replace("?","[?]{1}")
                   .replace("#","[#]{1}")
                   .replace(".","[.]{1}")
                   .replace(" ","[^+]*")
                   .replace("&","[^+]*")
                   .replace("and","[^+]*")
                   .replace("or","[^+]*")
                   .replace("/","[^+]*")
                   .replace(",","[^+]*")
                   .replace("-","[^+]*")
                   +"[^a-zA-Z0-9+]{1}",
                   Qt::CaseInsensitive
                   );

    if (autoreplace) {
        QString matchedSynonymText;
    //Loop over all technical synonym groups
        foreach (QStringList synonymGroup, technicalSynonyms) {
            bool synonymMatched=false;
        //Loop over all synonyms in the group
            foreach (QString s, synonymGroup) {
            //Check if any of synonyms matches the regexp
                if(QString(" "+s+" ").count(regexp)) {
                    synonymMatched=true;
                    matchedSynonymText=s.trimmed();
                    break;
                }
            }
            if (!synonymMatched)
                continue;

        //Loop again over all synonyms in the group
            foreach (QString s, synonymGroup) {
            //check each synonym to find it within resume field content and replace occurances with matched synonym
                if (s.trimmed()==matchedSynonymText)
                    continue;
                QString t=text();
                if (!t.count(s.trimmed(), Qt::CaseInsensitive))
                    continue;
                if (
                        (ask)
                        && (QMessageBox::question(
                                this,
                                "Auto Replace",
                                "Do you want to replace '"+s.trimmed()+"' with '"+matchedSynonymText+"'?\n"
                                +"in:\n"
                                +t,
                                QMessageBox::Ok | QMessageBox::Cancel,
                                QMessageBox::Cancel
                                ) != QMessageBox::Ok)
                        )
                    continue;
                setText(t.replace(s.trimmed(),matchedSynonymText, Qt::CaseInsensitive));
            }
        }
    }

//Search within field content
    count=QString(" "+text()+" ").count(regexp);

    if ((count) && (m_selectable) && (autoselect) && (!checkBox.isChecked()))
        checkBox.setChecked(true);

    return count;
}
