//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "groupbox.h"
#include "../../helpers/templator.h"
#include <QApplication>
#include <QDebug>

GroupBox::GroupBox(QWidget *parent,
                                     QString name,
                                     QString title,
                                     bool itemSelectable):
    QGroupBox(parent),
    vlayout(this,NULL,true,itemSelectable)
{
    connect(this,SIGNAL(toggled(bool)),this,SIGNAL(modified()));
    connect(&vlayout,SIGNAL(addButtonClicked()),this,SLOT(onAddButtonClicked()));
    connect(&vlayout,SIGNAL(insertButtonClicked(int)),this,SLOT(onInsertButtonClicked(int)));
    connect(&vlayout,SIGNAL(upButtonClicked()),this,SLOT(onUpButtonClicked()));
    connect(&vlayout,SIGNAL(downButtonClicked()),this,SLOT(onDownButtonClicked()));
    connect(&vlayout,SIGNAL(modified()),this,SIGNAL(modified()));
    setObjectName(name);
    setTitle(title);
    vlayout.checkBox.setText("Include "+title);
}

FieldsGroup *GroupBox::makeFieldsGroup()
{
    qWarning() << "makeFieldsGroup() is not implemented for class: " << metaObject()->className();
    return NULL;
}

QString GroupBox::exportDocX(QList<Translate *> templatetranslates)
{
    return vlayout.exportDocX(templatetranslates);
}

FieldsGroup *GroupBox::newFieldsGroup()
{
    FieldsGroup *fieldsGroup=makeFieldsGroup();
    connect(fieldsGroup,SIGNAL(insertClicked(int)),this,SLOT(onInsertButtonClicked(int)));
    connect(fieldsGroup,SIGNAL(modified()),this,SIGNAL(modified()));
    return fieldsGroup;
}

void GroupBox::onAddButtonClicked()
{
    vlayout.subgroupsLayout.addWidget(newFieldsGroup());
    emit modified();
}

void GroupBox::onInsertButtonClicked(int index)
{
    vlayout.subgroupsLayout.insertWidget(index,newFieldsGroup());
    emit modified();
}

void GroupBox::onUpButtonClicked()
{
    int index=this->parentWidget()->layout()->indexOf(this);
    if (index>0) {
        QVBoxLayout *parentLayout=qobject_cast<QVBoxLayout*>(this->parentWidget()->layout());
        parentLayout->removeWidget(this);
        parentLayout->insertWidget(index-1,this);
    }
    emit modified();
}

void GroupBox::onDownButtonClicked()
{
    int index=this->parentWidget()->layout()->indexOf(this);
    if (index<this->parentWidget()->layout()->count()-1) {
        QVBoxLayout *parentLayout=qobject_cast<QVBoxLayout*>(this->parentWidget()->layout());
        parentLayout->removeWidget(this);
        parentLayout->insertWidget(index+1,this);
    }
    emit modified();
}
