//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "scrollarea.h"
#include <QLabel>
#include <QApplication>
#include "field/abstract/fieldsgroup.h"
#include <QDebug>

ScrollArea::ScrollArea(QWidget *parent,
                                         QString name,
                                         QString description,
                                         bool itemSelectable) :
    QScrollArea(parent),
    vlayout(this,description,false,itemSelectable)
{
    QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    setSizePolicy(sizePolicy);
    setWidgetResizable(true);
    QWidget *w = new QWidget();
    w->setLayout(&vlayout);
    w->setObjectName(name);//Name here will be needed when vlayout tries to save its content to domElement
    connect(&vlayout,SIGNAL(addButtonClicked()),this,SLOT(onAddButtonClicked()));
    connect(&vlayout,SIGNAL(insertButtonClicked(int)),this,SLOT(onInsertButtonClicked(int)));
    connect(&vlayout,SIGNAL(modified()),this,SLOT(onModified()));
    setWidget(w);
    setObjectName(name);
}

FieldsGroup *ScrollArea::makeFieldsGroup()
{
    qWarning() << "makeFieldsGroup() is not implemented for class: " << metaObject()->className();
    return NULL;
}

void ScrollArea::onModified()
{
    emit modified();
}

FieldsGroup *ScrollArea::newFieldsGroup()
{
    FieldsGroup *fieldsGroup=makeFieldsGroup();
    connect(fieldsGroup,SIGNAL(insertClicked(int)),this,SLOT(onInsertButtonClicked(int)));
    connect(fieldsGroup,SIGNAL(modified()),this,SIGNAL(modified()));
    return fieldsGroup;
}

void ScrollArea::onAddButtonClicked()
{
    vlayout.subgroupsLayout.addWidget(newFieldsGroup());
    emit modified();
}

void ScrollArea::onInsertButtonClicked(int index)
{
    vlayout.subgroupsLayout.insertWidget(index,newFieldsGroup());
    emit modified();
}
