//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "jobapplication.h"
#include "abstract/field/multidatedcombofield.h"
#include "abstract/field/combofield.h"
#include "abstract/field/linefield.h"
#include "abstract/field/datefield.h"
#include "abstract/field/multilinefield.h"
#include "abstract/field/multidatedcombofield.h"
#include "definitions.h"

JobApplication::JobApplication(QWidget *parent) :
    FieldsGroup(parent,"jobapplication","Job Application",false,true)
{
    addField(new ComboField(this,
                                          "status",
                                          "Status",
                                          "Must Apply"+COMBO_ITEM_SEPARATOR
                                          +"Applied"+COMBO_ITEM_SEPARATOR
                                          +"Pending Me"+COMBO_ITEM_SEPARATOR
                                          +"Pending Them"+COMBO_ITEM_SEPARATOR
                                          +"Abandoned",
                                          false,
                                          false));
    addField(new LineField(this,"position","Position","Developer",true,false));
    addField(new LineField(this,"company","Company","Hyper Company",true,false));
    addField(new LineField(this,"location","Location","BigCity",false,false));
    addField(new DateField(this,"founddate","Date Found ("+QString(DATE_FORMAT)+")",NULL,false,false));
    addField(new LineField(this,"advertiseurl","Advertise URL",NULL,false,false));
    addField(new MultiLineField(this,"contacts","Follow-Up Contacts",NULL,false,false));
    addField(new MultiDatedComboField(this,"comunications","Attempts and Responses ("+QString(DATE_FORMAT)+")",
                                             "They Advertised"+COMBO_ITEM_SEPARATOR
                                             +"I Applied"+COMBO_ITEM_SEPARATOR
                                             +"Received thank you mail"+COMBO_ITEM_SEPARATOR
                                             +"I followed up"+COMBO_ITEM_SEPARATOR
                                             +"Arranged and appointment"+COMBO_ITEM_SEPARATOR
                                             +"Attended appointment"+COMBO_ITEM_SEPARATOR
                                             +"Sent thank you mail"+COMBO_ITEM_SEPARATOR
                                             +"Received sorry mail"+COMBO_ITEM_SEPARATOR
                                             +"Received job offer",false,false,false));
}

JobApplication::JobApplicationState JobApplication::jobApplicationState()
{
    QString status=static_cast<ComboField*>(fieldsLayout.itemAt(0)->widget())->text();
    if (
            (status=="Must Apply")
            || (status=="Pending Me")
            )
        return PendingMeCritical;
    else if (
             (status=="Applied")
             || (status=="Pending Them")
             ) {
        if (qobject_cast<MultiDatedComboField*>(fieldsLayout.itemAt(fieldsLayout.count()-1)->widget())->lastDate().daysTo(QDate::currentDate())<MAX_PENDING_DAYS)
            return OnGoing;
        else
            return PendingThemCritical;
    }
    else if (status=="Abandoned")
        return Abandoned;
    return Unknown;
}

void JobApplication::fold(bool fold)
{
    FieldsGroup::fold(fold);
    JobApplicationState s=jobApplicationState();
    switch (s) {
    case PendingMeCritical: {
            foldLabel.setStyleSheet("color: rgb(255, 0, 0)"+COMBO_ITEM_SEPARATOR);
            foldLabel.setText("<b>"+foldLabel.text()+"</b>");
    }; break;
    case OnGoing: foldLabel.setStyleSheet("color: rgb(0, 128, 0)"+COMBO_ITEM_SEPARATOR); break;
    case PendingThemCritical: {
        foldLabel.setStyleSheet("color: rgb(255, 128, 0)"+COMBO_ITEM_SEPARATOR);
        foldLabel.setText("<b>"+foldLabel.text()+"</b>");
    }; break;
    case Abandoned: foldLabel.setStyleSheet("color: rgb(128, 128, 128)"+COMBO_ITEM_SEPARATOR); break;
    case Unknown: break;
    }
}

QString JobApplication::exportCSV()
{
    QString content=static_cast<InputField*>(fieldsLayout.itemAt(1)->widget())->text().replace("\t"," ")
            +"\t"+static_cast<InputField*>(fieldsLayout.itemAt(2)->widget())->text().replace("\t"," ")
            +"\t"+static_cast<InputField*>(fieldsLayout.itemAt(3)->widget())->text().replace("\t"," ")
            +"\t"+static_cast<InputField*>(fieldsLayout.itemAt(4)->widget())->text().replace("\t"," ")
            +"\t"+static_cast<InputField*>(fieldsLayout.itemAt(5)->widget())->text().replace("\t"," ")
            +"\n";
    foreach (QString s, static_cast<MultiLineField*>(fieldsLayout.itemAt(6)->widget())->texts())
        content+="\t\t\t\t\t"+s+"\n";
    foreach (QString s, static_cast<MultiLineField*>(fieldsLayout.itemAt(7)->widget())->texts())
        content+="\t\t\t\t\t\t"+s+"\n";
    return content;
}
