//    CVAssistant
//    Copyright 2015-2016 Aario Shahbany
//
//    This file is part of CVAssistant.
//
//    CVAssistant is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    CVAssistant is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with CVAssistant.  If not, see <http://www.gnu.org/licenses/>.

#include "resumedocument.h"
#include <QFile>
#include <QDebug>
#include <QProcess>
#include <QDateTime>
#include <QDesktopServices>
#include <QMessageBox>
#include <QUrl>
#define APP_URL     "http://cvassistant.sf.net"

void ResumeDocument::saveFile(QuaZip *zip, QString filename, QString content)
{
    QuaZipFile zipFile(zip);
    zipFile.open(QIODevice::WriteOnly | QIODevice::Text, QuaZipNewInfo(filename));
    zipFile.write(content.toUtf8().data(), content.toUtf8().size());
    zipFile.close();
}

QString ResumeDocument::formatItems(QStringList items, QString templateName, QString itemTag)
{
    QString result;
    for (int i=0; i<items.count(); i++) {
        QString templateString=Templator::readTemplate(templateName,templatetranslates);
        result.append(templateString.replace(itemTag,items.at(i).toHtmlEscaped()));
    }
    return result;
}

bool ResumeDocument::removeDir(const QString &dirName)

{
    bool result = true;
    QDir dir(dirName);

    if (dir.exists(dirName)) {
        Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) {
            if (info.isDir()) {
                result = removeDir(info.absoluteFilePath());
            }
            else {
                result = QFile::remove(info.absoluteFilePath());
            }

            if (!result) {
                return result;
            }
        }
        result = dir.rmdir(dirName);
    }
    return result;
}

ResumeDocument::ResumeDocument(QList<Translate *> translates) :
    templatetranslates(translates),
    pmProfilePhoto(NULL)
{
}

void ResumeDocument::addProfilePhoto(QPixmap *profilePhoto)
{
    pmProfilePhoto=profilePhoto;
    content.append(Templator::readTemplate("profilephoto",templatetranslates)
                   .replace(
                       "[profilephoto_height]",
                       QString().number((quint64)1260000*pmProfilePhoto->height()/pmProfilePhoto->width())
                       )
                   );
}

void ResumeDocument::save(QString filename, int fontSize)
{
    if (!filename.endsWith(".docx"))
        filename=filename+".docx";
    QuaZip zip(filename);
    zip.open(QuaZip::mdCreate);
    saveFile(&zip,"_rels/.rels",Templator::readTemplate("dotrels",templatetranslates));
    saveFile(&zip,"docProps/app.xml",Templator::readTemplate("app",templatetranslates).replace("[paragraphs]",QString::number(content.count("<w:p>"))));
    saveFile(&zip,"docProps/core.xml",
             Templator::readTemplate("core",templatetranslates)
             .replace("[created]",QDateTime::currentDateTime().toString("yyyy-MM-ddThh:mm:ssZ"))
             .replace("[modified]",QDateTime::currentDateTime().toString("yyyy-MM-ddThh:mm:ssZ")));
    saveFile(&zip,"word/_rels/document.xml.rels",Templator::readTemplate("document.xml.rels",templatetranslates));
    saveFile(&zip,"word/fontTable.xml",Templator::readTemplate("fontTable",templatetranslates));
    saveFile(&zip,"word/numbering.xml",Templator::readTemplate("numbering",templatetranslates));
    saveFile(&zip,"word/settings.xml",Templator::readTemplate("settings",templatetranslates));
    saveFile(&zip,"word/styles.xml",Templator::readTemplate("styles",templatetranslates).replace("[font_size]",QString::number(fontSize*2)));
    saveFile(&zip,"[Content_Types].xml",Templator::readTemplate("[Content_Types]",templatetranslates));
    if (pmProfilePhoto != NULL) {
        QuaZipFile zipFile(&zip);
        zipFile.open(QIODevice::WriteOnly, QuaZipNewInfo("word/media/image1.jpeg"));
        pmProfilePhoto->save(&zipFile,"JPEG");
        zipFile.close();
    }
    saveFile(&zip,"word/document.xml",Templator::readTemplate("document",templatetranslates).replace("[document_body]",content));

    zip.close();
    QDesktopServices::openUrl(QUrl::fromLocalFile(filename));
    //That's it!
}
